/*
 * Original code Copyright 2002 Eric Hodel. All rights reserved.
 * The rest Copyright 2003+ David Murray/Robert Strong
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of Eric Hodel nor the names of his
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ERIC HODEL AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ERIC HODEL OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

// Yes, most of these functions are from Livelizard (slightly modified) unless otherwise stated.

var ub = document.getElementById("utilbundle");

const AppName = "Mozilla-DeepestSender/0.5.0";
const PASSWORD_CID = "@mozilla.org/passwordmanager;1";
const PREF_CID = "@mozilla.org/preferences;1";

var gPasswordManager;
var gPasswordManagerInternal;
var gPreferences;


function getPrefDS() {
  if (!gPreferences) {
    try {
      gPreferences = Components.classes[PREF_CID].getService();
      gPreferences = gPreferences.QueryInterface(
        Components.interfaces.nsIPref);
    } catch (e) {}
  }

  return gPreferences;
}

function getPasswordManager() {
  if (!gPasswordManager) {
    try {
      gPasswordManager = Components.classes[PASSWORD_CID].getService();
      gPasswordManager = gPasswordManager.QueryInterface(
        Components.interfaces.nsIPasswordManager);
    } catch (e) {
      dump("getPasswordManager() failed\n");
    }
  }

  return gPasswordManager;
}

function getPasswordManagerInternal() {
  if (!gPasswordManagerInternal) {
    try {
      gPasswordManagerInternal =
        Components.classes[PASSWORD_CID].getService();
      gPasswordManagerInternal = gPasswordManagerInternal.QueryInterface(
        Components.interfaces.nsIPasswordManagerInternal);
    } catch (e) {
      dump("getPasswordManagerInternal() failed\n");
    }
  }

  return gPasswordManagerInternal;
}

function getPassword(loginData) {
  if (!loginData)
    return null;

  var pmInternal = getPasswordManagerInternal();
  if (!pmInternal)
    return null;

  var host = {value:''};
  var user = {value:''};
  var password = {value:''};

  try {
    pmInternal.findPasswordEntry(loginData.LJURL, '', '',
      host, user, password);
    loginData.username = user.value;
    loginData.password = password.value;
    return loginData;
  } catch(e) {
    dump("findPasswordEntry() failed\n");
  }

  return null;
}

function savePassword(loginData) {
  if (!loginData || !loginData.LJURL || !loginData.username)
    return false;

  var pm = getPasswordManager();

  if (!pm)
    return false;

  try {
    pm.removeUser(loginData.LJURL, loginData.username);
  } catch(e) {
    dump("removeUser() failed\n");
  }

  if (loginData.savePassword) {
    try {
      pm.addUser(loginData.LJURL, loginData.username, loginData.password);
    } catch(e) {
      dump("addUser failed\n");
    }

    return true;
  }
  
  return false;
}

function LoadCredentials() {
  var loginData = {
    LJURL: LJURL,
    username : '',
    password : ''
  };
 
  loginData = getPassword(loginData);
  if (!loginData)
    return false;

  document.getElementById("username").value = loginData.username;
  document.getElementById("password").value = loginData.password;

  return true;
}

function SaveCredentials(username, password, savepass) {
  var loginData = {
    LJURL : LJURL,
    username : username,
    password : password,
    savePassword : savepass
  };

  return savePassword(loginData);
}

function getResponseData(heading, responseText) {
	var i = 0;
	var data = null;

	try {
		var responseData = responseText.split("\n");
	} catch(e) {
		return "";
	}

	while (i != responseData.length) {
		if (responseData[i] == heading) {
			data = responseData[i + 1];
			return data;
		}
	i++;
}

  return null;
}

// Grabbed this off the net. It's just to get rid of any whitespace at the start and end of strings, to stop people from just pressing space a few times then
// hitting post to send a blank entry off to LJ.
function strtrim() {
	return this.replace(/^\s+/,'').replace(/\s+$/,'');
}
String.prototype.trim = strtrim;

// This next stuff is by me.
function FindIndex(arrayname,searchin,searchfor) {
	// This is to set the selectedIndex of something to whatever the first thing that matches the tag is. Very handy.
	for (i=0;i<arrayname.childNodes.length;i++) {
		if (arrayname.childNodes[i].getAttribute(searchin) == searchfor) {
			return i;
			break;
		}
	}
	return -1;
}

function SetDefaultPrefs() {
	// These are just here because I'm not sure where the best spot for them is yet.
	getPrefDS().SetDefaultBoolPref("deepestSender.OpenInSidebar", false);
	getPrefDS().SetDefaultCharPref("deepestSender.PicAlign", "center");
	getPrefDS().SetDefaultBoolPref("deepestSender.UseCSS",false);
	getPrefDS().SetDefaultBoolPref("deepestSender.RemoveEmptyTags",false);
	getPrefDS().SetDefaultBoolPref("deepestSender.RemoveMozAttrs",false);
	getPrefDS().SetDefaultBoolPref("deepestSender.SpellCheckOnPost",false);
	getPrefDS().SetDefaultBoolPref("deepestSender.AutoDetectMusic",false);
	getPrefDS().SetDefaultBoolPref("deepestSender.SourceViewDefault",false);
}


////////////////////////////////////////////////////////////////////////////////
// txtFilePicker
//
// Parameters:
//  aTitle: title to go on file picker window
//  aSave: 1 if picking file to save/overwrite, 0 if picking file to load
//  aStart: directory to start from
//    This must be an nsILocalFile.  new Dir("foo") in jslib/io/dir.js will
//    do the trick, but you have to do it yourself.
// Returns:
//  Name of file picked, in URL format, or null if cancelled
// Original taken and modified from Tagzilla.mozdev.org!
////////////////////////////////////////////////////////////////////////////////
function FilePicker(aTitle, aSave) {
  var retVal = null;
  try {
   
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    fp.init(window, aTitle, (aSave ? nsIFilePicker.modeSave : nsIFilePicker.modeOpen));
    fp.appendFilters(nsIFilePicker.filterText);
	fp.appendFilters(nsIFilePicker.filterAll);
    
    var result=fp.show();
    if (result == nsIFilePicker.returnOK || result == nsIFilePicker.returnReplace) {
      retVal=fp.file.path;
    }
  }
  catch (ex) {
  }
  return retVal;
}

// Sends off request to LiveJournal, all nice and asynchronous so it doesn't freeze up Mozilla.
function SendRequest(mode, URL, whattosend, ReadyStateFunction, ErrorFunction) {
	request = new XMLHttpRequest()
	request.onload = ReadyStateFunction;
	request.onerror = ErrorFunction;
    request.open(mode, URL, true);
	request.send(whattosend);
}

// Open post options window...
function PostOptionsWindow() {
	window.openDialog("chrome://deepestsender/content/PostOptions.xul","dspostoptions","chrome,modal",result);
}

function OpenHistoryWindow() {
	window.openDialog("chrome://deepestsender/content/dsHistory.xul","dsHistory","chrome,modal");
}

function EditLastEntry() {
	window.openDialog("chrome://deepestsender/content/EditEntry.xul","editentry","chrome,modal,resizable",-1);
}

function decodeLJText(ljtext) {
	if (ljtext) {
		ljtext = ljtext.replace(/\+/gi, "%20");
		ljtext = decodeURIComponent(ljtext);
	} else {
		ljtext = "";
	}
	return ljtext;
}

// For when the userpic is clicked, this opens the user's LJ in the most recently opened Window (or in the main content window if DS is in the sidebar.
// It'll also open a new window if theres no browser window opened. Found this by digging in the Mozilla source; much handier than my crappy old way.

function OpenJournalPage(extra) {
	var lj = LJPage + "/users/";
    if (username != usejournal) {
		lj += usejournal;
	} else {
		lj += username;
	}
	if (extra) {
		 lj += extra;
	}
	try {
		openTopWin(lj);
	} catch (e) {
		// All this crap is to make it so Mozilla doesn't go weird with its open windows when trying to switch between them...
		try {
			opener.opener.open(lj,"ljview");
		} catch (e) {
			try {
				opener.open(lj,"ljview");
			} catch (e) {
				window.open(lj,"ljview");
			}
		}
	}
}

function DisableKeys(setkey) {
	var keyset = document.getElementById("deepestsendercommands");
	for (i = 11; i < keyset.childNodes.length; i++) {
		keyset.childNodes[i].setAttribute("disabled", setkey);
	}
		// Spellchecker.
	if (!IsSpellCheckerInstalled()) {
		document.getElementById("cmd_spell").setAttribute("disabled",true);
	}
}

function ZeroFix(TheNumber) {
	// To pad out numbers so they're always 2 digits long.
	if (TheNumber < 10) { TheNumber = "0" + TheNumber; }
	return TheNumber;
}

function SpellCheck() {
	window.openDialog("chrome://editor/content/EdSpellCheck.xul", "_blank","chrome,close,titlebar,modal", false, false, true);

}

function IsSpellCheckerInstalled() {
	return "@mozilla.org/spellchecker;1" in Components.classes;
}

function DetectMusic() {
	var fileloc = getPrefDS().CopyCharPref("deepestSender.MP3InfoFile");
	if (fileloc) {
		var mp3string = getPrefDS().CopyCharPref("deepestSender.MP3Display");
		var musicfile = new File(fileloc);
		if (musicfile.exists()) {
			musicfile.open();
			var searcher, musicstring;
			musicstring = musicfile.read();
			musicfile.close();
			musicstring = musicstring.replace(/\r\n/gi, "\n");
			musicstring = musicstring.replace(/\r/gi, "\n");
			var musicarray = [];
			musicarray = musicstring.split("\n")
			for (i = 0; i < musicarray.length; i++) {
				searcher = new RegExp("\%" + (i + 1) + "\%","gi");
				mp3string = mp3string.replace(searcher,musicarray[i]);
			}
			if (mp3string.trim().length > 3) {
				document.getElementById("music").value = mp3string.trim();
			} else {
				document.getElementById("music").value = "";
			}
		} else {
			alert(ub.getString("MP3InfoNotExist"));
		}
	} else {
		if (confirm(ub.getString("MP3NotSetUp"))) {
			window.openDialog("chrome://deepestsender/content/MP3Config.xul","mp3settings","chrome,modal");
		}
	}
}

function ShowSecurity() {
	// To open up the Post Security Options. Note to self: combine these window opening things into just the one function.
	window.openDialog("chrome://deepestsender/content/SecurityOptions.xul","securityoptions","chrome,modal",result);
}

// Returns isCommandEnabled for a command
function CommandStatus(command) {
	try {
		var controller = top.document.commandDispatcher.getControllerForCommand(command);
		var enabled = false;
		if ( controller )
			enabled = controller.isCommandEnabled(command);

		return (enabled);
	}
	catch (e) {
		dump("An error occurred updating the "+command+" command\n");
	}
}
function goSetCommandEnabled(id, enabled) {
	var node = document.getElementById(id);
	if ( node ) {
		if ( enabled )
			node.removeAttribute("disabled");
		else
			node.setAttribute('disabled', 'true');
	}
}

// update menu items that rely on focus
function goUpdateGlobalEditMenuItems() {
	goUpdateCommand('cmd_undo');
	goUpdateCommand('cmd_redo');
	goUpdateCommand('cmd_cut');
	goUpdateCommand('cmd_copy');
	goUpdateCommand('cmd_paste');
	goUpdateCommand('cmd_pasteNoFormatting');
	goUpdateCommand('cmd_selectAll');
	goUpdateCommand('cmd_delete');
}

// update menu items that rely on the current selection
function goUpdateSelectEditMenuItems() {
	goUpdateCommand('cmd_cut');
	goUpdateCommand('cmd_copy');
	goUpdateCommand('cmd_delete');
	goUpdateCommand('cmd_selectAll');
}

// update menu items that relate to undo/redo
function goUpdateUndoEditMenuItems() {
	goUpdateCommand('cmd_undo');
	goUpdateCommand('cmd_redo');
}

// update menu items that depend on clipboard contents
function goUpdatePasteMenuItems() {
	goUpdateCommand('cmd_paste');
	goUpdateCommand('cmd_pasteNoFormatting');
}

function setNormalControls() {
	goSetCommandEnabled("menu_Edit", true);
	goSetCommandEnabled("style_menu", true);
}
function setSourceControls() {
	goSetCommandEnabled("menu_Edit", true);
	goSetCommandEnabled("style_menu", false);
}
function setPreviewControls() {
	goSetCommandEnabled("menu_Edit", false);
	goSetCommandEnabled("style_menu", false);

}
// Checks status of the edit commands in the context menu and sets disabled as appropriate
function goUpdateContextMenuItems() {
	var enabled = false;
	enabled = CommandStatus("cmd_undo")
	goSetCommandEnabled("cMenu_undo", enabled);

	enabled = CommandStatus("cmd_redo")
	goSetCommandEnabled("cMenu_redo", enabled);

	enabled = CommandStatus("cmd_cut")
	goSetCommandEnabled("cMenu_cut", enabled);

	enabled = CommandStatus("cmd_copy")
	goSetCommandEnabled("cMenu_copy", enabled);

	enabled = CommandStatus("cmd_paste")
	goSetCommandEnabled("cMenu_paste", enabled);

	enabled = CommandStatus("cmd_pasteNoFormatting")
	goSetCommandEnabled("cMenu_pasteNoFormatting", enabled);

	enabled = CommandStatus("cmd_delete")
	goSetCommandEnabled("cMenu_delete", enabled);

	enabled = CommandStatus("cmd_selectAll")
	goSetCommandEnabled("cMenu_selectAll", enabled);
}

//More stuff ripped out of utilityOverlay.js. Stupid Firefox devs.

function getTopWin() {
    var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
    var windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
    var topWindowOfType = windowManagerInterface.getMostRecentWindow( "navigator:browser" );

    if (topWindowOfType) {
        return topWindowOfType;
    }
    return null;
}

function openTopWin( url ) {
    /* note that this chrome url should probably change to not have
       all of the navigator controls, but if we do this we need to have
       the option for chrome controls because goClickThrobber() needs to
       use this function with chrome controls */
    /* also, do we want to
       limit the number of help windows that can be spawned? */
    if ((url == null) || (url == "")) return null;

    // xlate the URL if necessary
    if (url.indexOf("urn:") == 0) {
        url = xlateURL(url);        // does RDF urn expansion
    }

    // avoid loading "", since this loads a directory listing
    if (url == "") {
	url = "about:blank";
    }

    var topWindowOfType = getTopWin();
    if ( topWindowOfType ) {
        topWindowOfType.focus();
        topWindowOfType.loadURI(url);
        return topWindowOfType;
    }
    return window.openDialog( getBrowserURL(), "_blank", "chrome,all,dialog=no", url );
}